var preloadScripts = [];
var allScripts = [];
var inlineScripts = [];
var plainScripts = [];
var isTouchDevice = 'ontouchstart' in window || navigator.msMaxTouchPoints;
var preloadRunned = false;

window.addEventListener('load', function () {
    var scrollTop = window.scrollY;
    if (scrollTop > 60) {
        preload();
    }
});

function preload() {
    var iframes = [].slice.call(document.querySelectorAll("iframe.wpc-iframe-delay"));
    var styles = [].slice.call(document.querySelectorAll('[rel="wpc-stylesheet"],[type="wpc-stylesheet"]'));
    var mobileStyles = [].slice.call(document.querySelectorAll('[rel="wpc-mobile-stylesheet"],[type="wpc-mobile-stylesheet"]'));

    var customPromiseFlag = [];

    styles.forEach(function (element, index) {
        var promise = new Promise(function (resolve, reject) {
            if (element.tagName.toLowerCase() === 'link') {
                element.setAttribute('rel', 'stylesheet');
            }
            element.setAttribute('type', 'text/css');
            element.addEventListener('load', function () {
                resolve();
            });

            element.addEventListener('error', function () {
                reject();
            });
        });
        customPromiseFlag.push(promise);
    });

    styles = [];

    iframes.forEach(function (element, index) {
        var promise = new Promise(function (resolve, reject) {
            var iframeUrl = element.getAttribute('data-src');
            element.setAttribute('src', iframeUrl);
            element.addEventListener('load', function () {
                resolve();
            });

            element.addEventListener('error', function () {
                reject();
            });
        });
        customPromiseFlag.push(promise);
    });

    iframes = [];

    mobileStyles.forEach(function (element, index) {
        var promise = new Promise(function (resolve, reject) {
            if (element.tagName.toLowerCase() === 'link') {
                element.setAttribute('rel', 'stylesheet');
            }
            element.setAttribute('type', 'text/css');
            element.addEventListener('load', function () {
                resolve();
            });

            element.addEventListener('error', function () {
                reject();
            });
        });
        customPromiseFlag.push(promise);
    });

    mobileStyles = [];

    Promise.all(customPromiseFlag).then(function () {
        var criticalCss = document.querySelector('#wpc-critical-css');
        if (criticalCss) {
            criticalCss.remove();
        }
    }).catch(function () {
        styles.forEach(function (element, index) {
            if (element.tagName.toLowerCase() === 'link') {
                element.setAttribute('rel', 'stylesheet');
            }
            element.setAttribute('type', 'text/css');
        });
    });

    window.removeEventListener('load', preload);
}

function mouseMoveFun() {
    allScripts = [].slice.call(document.querySelectorAll('script[type=\"wpc-delay-script\"]'));
    window.removeEventListener('mousemove', mouseMoveFun);

    // Remove Preloader
    const aFPreloader = document.getElementById('af-preloader');
    if (aFPreloader) {
        aFPreloader.remove();
    }

    inlineEval();
    loadJs();
    preload();
}

window.addEventListener('mousemove', mouseMoveFun);

if (isTouchDevice) {
    window.addEventListener('touchstart', mouseMoveFun);
}

function preloadScriptsFunc() {
    inlineScripts = [].slice.call(document.querySelectorAll('script[type=\"wpc-inline-delay-script\"]'));
    allScripts = [].slice.call(document.querySelectorAll('script[type=\"wpc-delay-script\"]'));

    window.removeEventListener('DOMContentLoaded', preloadScriptsFunc);

    if (inlineScripts.length > 0 || allScripts.length > 0) {
        startPreloading();
    }
}

preloadScriptsFunc();


function startPreloadingNext() {
    setTimeout(function () {
        startPreloading();
    }, 10);
    return;
}

function startPreloading() {
    if (allScripts.length === 0) {
        return true;
    }

    element = allScripts[0];
    allScripts.splice(0, 1);
    if (!element.hasAttribute('src') && !element.id.includes('-before') && !element.id.includes('-after') && !element.id.includes('-extra')) {
        // Inlined
    } else {
        if (element !== null) {
            var src = element.getAttribute('src');
            if (src !== null) {
                if (!src.includes('gtag') && !src.includes('googletag')) {
                    var new_element = document.createElement('link');
                    new_element.setAttribute('rel', "preload");
                    new_element.setAttribute('href', src);
                    new_element.setAttribute('as', 'script');
                    document.head.appendChild(new_element);
                } else {
                    // Gtag Load Instantly but Defer
                    element.setAttribute('type', 'text/javascript');
                    element.defer = true;
                    element.async = false;
                    allScripts.shift();
                }
            }
        }
    }

    startPreloadingNext();
    return true;
}


var loadJsRunning = false;

function loadJsNext() {
    if (allScripts.length === 0) {
        window.document.dispatchEvent(new Event("DOMContentLoaded"));
        window.document.dispatchEvent(new Event("resize"));

        setTimeout(function() {
            var slider = document.getElementsByClassName('banner-image');
            if (slider !== null && slider.length !== 0 && slider !== undefined) {
                // Iterate through each element using a for loop
                for (var i = 0; i < slider.length; i++) {
                    // Apply changes to each element
                    slider[i].style.display = 'block';
                }
            }

            var gForm = document.getElementsByClassName('gform_wrapper');
            if (gForm !== null && gForm.length !== 0 && gForm !== undefined) {
                // Iterate through each element using a for loop
                for (var i = 0; i < gForm.length; i++) {
                    // Apply changes to each element
                    gForm[i].style.display = 'block';
                }
            }
        },300);

    } else {
        loadJs();
    }
}


function inlineEval() {
    inlineScripts = [].slice.call(document.querySelectorAll('script[type=\"wpc-inline-delay-script\"]'));

    if (inlineScripts.length === 0) {
        return true;
    }

    var element = inlineScripts[0];
    var elementID = element.id;
    inlineScripts.splice(0, 1);

    try {
        (function() {
            eval(element.textContent);
        })();

        // Attempt to run the code using eval
        var new_element = document.createElement('script');
        new_element.textContent = element.textContent;
        new_element.setAttribute('type', 'text/javascript');
        new_element.async = false;

        whereToAppend(element, new_element);

        element.remove();
        inlineEval();
    } catch (error) {
        // If eval throws an error, log "error" along with the error message
    }
}

function loadInlinedJs() {
    inlineScripts = [].slice.call(document.querySelectorAll('script[type=\"wpc-inline-delay-script\"]'));

    if (inlineScripts.length === 0) {
        return true;
    }

    var element = inlineScripts[0];
    var elementID = element.id;

    inlineScripts.splice(0, 1);

    var new_element = document.createElement('script');
    new_element.textContent = element.textContent;
    new_element.setAttribute('type', 'text/javascript');
    new_element.async = false;

    whereToAppend(element, new_element);

    element.remove();
    loadInlinedJs();

    return true;
}


function whereToAppend(originalElement, newElement) {
    var isInHead = false;
    //var isInHead = document.head.contains(originalElement);
    if (isInHead) {
        //document.head.appendChild(newElement);
    } else {
        document.body.appendChild(newElement);
    }
}


function loadJs() {

    if (allScripts.length === 0) {
        window.document.dispatchEvent(new Event("DOMContentLoaded"));
        window.document.dispatchEvent(new Event("resize"));
        return true;
    }

    var element = allScripts[0];
    var elementID = element.id;

    allScripts.splice(0, 1);

    // External script
    var jsBefore = document.getElementById(elementID + '-before');
    var jsAfter = document.getElementById(elementID + '-after');
    var jsExtra = document.getElementById(elementID + '-extra');

    if (jsBefore !== null) {
        var newElementBefore = document.createElement('script');
        newElementBefore.textContent = jsBefore.textContent;
        newElementBefore.setAttribute('type', 'text/javascript');
        newElementBefore.async = false;

        whereToAppend(jsBefore, newElementBefore);

        jsBefore.remove();
    }


    if (jsExtra !== null) {
        var newElementExtra = document.createElement('script');
        newElementExtra.textContent = jsExtra.textContent;
        newElementExtra.setAttribute('type', 'text/javascript');
        newElementExtra.async = false;

        whereToAppend(jsExtra, newElementExtra);
        jsExtra.remove();
    }

    if (element !== null) {
        if (element.getAttribute('src') !== null) {

            console.log("Loading JS: " + element.getAttribute('src'));

            var new_element = document.createElement('script');
            var elementId = element.getAttribute('id');
            new_element.setAttribute('src', element.getAttribute('src'));
            new_element.setAttribute('id', elementId);
            new_element.setAttribute('type', 'text/javascript');
            new_element.async = false;

            // Set up onload event for modern browsers
            new_element.onload = function () {
                console.log("Loaded JS: " + element.getAttribute('src'));
                loadAfterJs(elementId);

                var leftoverScripts = [].slice.call(document.querySelectorAll('script[type=\"wpc-delay-script\"]'));
                if (leftoverScripts.length <= 1) {
                    loadInlinedJs();
                }

                window.document.dispatchEvent(new Event("DOMContentLoaded"));
                window.document.dispatchEvent(new Event("resize"));
                loadJsNext();
                resolve(true);
            };

            whereToAppend(element, new_element);
            element.remove();
        } else {
            if (element.id.includes('-after')) {
                // Nothing
                loadJsNext();
            } else {
                var new_element = document.createElement('script');
                new_element.textContent = element.textContent;
                new_element.setAttribute('type', 'text/javascript');
                new_element.async = false;

                whereToAppend(element, new_element);
                element.remove();
                loadJsNext();
            }
        }

        return true;
    }

    loadJsNext();
    return true;
}


function loadAfterJs(elementId) {
    var jsAfter = document.getElementById(elementId + '-after');
    if (jsAfter !== null) {
        var content = jsAfter.textContent;
        var id = jsAfter.getAttribute('id');
        var newElementAfter = document.createElement('script');

        newElementAfter.textContent = content;
        newElementAfter.setAttribute('type', 'text/javascript');
        newElementAfter.setAttribute('id', id);
        newElementAfter.async = false;

        whereToAppend(jsAfter, newElementAfter);
        jsAfter.remove();
    }
}