<?php

class wps_ic_preload_warmup
{

  private static $apiUrl;

  public function __construct() {
    $warmupVersion = 'v2/';
    $location = get_option('wps_ic_geo_locate');
    if (empty($location)) {
      $location = $this->geoLocate();
    }

    if (is_object($location)) {
      $location = (array)$location;
    }

    if (isset($location) && !empty($location)) {
      if (is_array($location) && !empty($location['server'])) {
        if ($location['continent'] == 'CUSTOM') {
          self::$apiUrl = 'https://' . $location['custom_server'] . '.zapwp.net/warmup/';
        } elseif ($location['continent'] == 'AS' || $location['continent'] == 'IN') {
          self::$apiUrl = 'https://singapore.zapwp.net/warmup/';
        } elseif ($location['continent'] == 'EU') {
          self::$apiUrl = 'https://germany.zapwp.net/warmup/';
        } elseif ($location['continent'] == 'OC') {
          self::$apiUrl = 'https://sydney.zapwp.net/warmup/';
        } elseif ($location['continent'] == 'US' || $location['continent'] == 'NA' || $location['continent'] == 'SA') {
          self::$apiUrl = 'https://nyc.zapwp.net/warmup/';
        } else {
          self::$apiUrl = 'https://germany.zapwp.net/warmup/';
        }
      } else {
        self::$apiUrl = 'https://' . $location->server . '/warmup/';
      }
    } else {
      self::$apiUrl = 'https://germany.zapwp.net/warmup/';
    }

    self::$apiUrl .= $warmupVersion;

  }

  public function geoLocate() {
    $force_location = get_option('wpc-ic-force-location');
    if (!empty($force_location)) {
      return $force_location;
    }

    $call = wp_remote_get('https://cdn.zapwp.net/?action=geo_locate&domain=' . urlencode(site_url()), array('timeout' => 30, 'sslverify' => false, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0'));
    if (wp_remote_retrieve_response_code($call) == 200) {
      $body = wp_remote_retrieve_body($call);
      $body = json_decode($body);

      if ($body->success) {
        update_option('wps_ic_geo_locate', $body->data);

        return $body->data;
      } else {
        update_option('wps_ic_geo_locate', array('country' => 'EU', 'server' => 'frankfurt.zapwp.net'));

        return array('country' => 'EU', 'server' => 'frankfurt.zapwp.net');
      }
    } else {
      update_option('wps_ic_geo_locate', array('country' => 'EU', 'server' => 'frankfurt.zapwp.net'));

      return array('country' => 'EU', 'server' => 'frankfurt.zapwp.net');
    }
  }

  public function getPagesForFiltering($post_type, $post_status, $page_number, $offset, $search = '') {
    $pages = $this->getPages($post_type, 1, 0, -1, $search);
    $wpc_excludes = get_option('wpc-excludes');
    $url_key_class = new wps_ic_url_key();
    $settings = get_option(WPS_IC_SETTINGS);
    $total_count = 0;
    $filtered_pages = [];
    $start_index = ($page_number - 1) * 10;
    $end_index = $start_index + 10;

    //local addition
    $local = get_option('wpc-connectivity-status');
    //end local addition
    foreach ($pages as $key => $page) {
      $url = $page['link'];
      $urlKey = $url_key_class->setup($url);
      $cachePath = WPS_IC_CACHE . $urlKey . '/';
      $critPath = WPS_IC_CRITICAL . $urlKey . '/critical_desktop.css';
      $page_excludes = isset($wpc_excludes['page_excludes'][$page['id']]) ? $wpc_excludes['page_excludes'][$page['id']] : array();
      $cacheActive = (!empty($settings['cache']['advanced']) && $settings['cache']['advanced'] == '1' && !isset($page_excludes['advanced_cache'])) || (isset($page_excludes['advanced_cache']) && $page_excludes['advanced_cache'] == '1');

      if (!empty($page['errors'])) {
        $hasErrorCode = false;
        foreach ($page['errors'] as $errorCode) {
          if (is_numeric($errorCode) && (int)$errorCode >= 300 && (int)$errorCode <= 600) {
            $hasErrorCode = true;
            break;
          }
        }
      }

      $cacheGenerated = '0';
      $critGenerated = '1';
      if ($cacheActive) {
        if (function_exists('gzencode')) {
          if (file_exists($cachePath . 'index.html' . '_gzip') && filesize($cachePath . 'index.html' . '_gzip') > 0) {
            $cacheGenerated = '1';
          }
        } else {
          if (file_exists($cachePath . 'index.html') && filesize($cachePath . 'index.html') > 0) {
            $cacheGenerated = '1';
          }
        }
      }

      $doNotCache = false;
      if (!empty($page['errors']['notice'])) {
        $doNotCache = in_array('DONOTCACHEPAGE', $page['errors']['notice']);
      }

      //local addition
      if (!empty($local) && $local == 'failed' && ($cacheGenerated == '1' || $doNotCache)) {
        $preloaded = '1';
      } else {
        $preloaded = '1';

        //check critical
        if ((isset($page_excludes['critical_css']) && $page_excludes['critical_css'] == '0')) {
          // Excluded from Smart Optimizations
        } else if (!empty($settings['critical']['css']) && $settings['critical']['css'] == '1') {
          if (!file_exists($critPath)) {
            $critGenerated = '0';
            $preloaded = '0';
          }
        }

        if (($cacheGenerated == '0' && !$doNotCache) && $cacheActive) {
          $preloaded = '0';
        }
      }

      $page['preloaded'] = $preloaded;
      $page['critGenerated'] = $critGenerated;
      $page['cacheGenerated'] = $cacheGenerated;

      $optimized = $preloaded === '1';
      $skipped = isset($hasErrorCode) && $hasErrorCode;

      $current_status = $skipped ? 'skipped' : ($optimized ? 'optimized' : 'unoptimized');

      if (in_array($current_status, $post_status)) {
        $total_count++;

        if ($total_count > $start_index && $total_count <= $end_index) {
          $filtered_pages[] = $page;
        }
      }
    }

    // Return the total count and the filtered subset of pages
    $return = ['total' => $total_count, 'pages' => $filtered_pages];

    return $return;
  }

  public function getPages($post_type, $page, $offset, $limit, $search = '') {
    $available_pages = [];
    $exclude_id = [];

    $post_info = get_transient('wpc-post-info', []);
    $warmup_errors = get_option('wpc-warmup-errors', []);

    if ($post_type == 'any' || empty($post_type)) {
      $post_type = ['post', 'page', 'product'];
    }

    $homePage = get_option('page_on_front');
    $postsPage = get_option('page_for_posts');

    if (!empty($homePage)) {
      if ($page == 1 && (in_array('page', $post_type))) {
        if (!isset($post_info[$homePage])) {
          $post_info[$homePage] = $this->fetchPostInfo($homePage, ' (Home Page)');
          $post_info[$homePage]['home'] = true;
        }

        if (!empty($search)) {
          if (stripos($post_info[$homePage]['title'], $search) !== false) {
            $available_pages[] = $post_info[$homePage];
          }
        } else {
          $available_pages[] = $post_info[$homePage];
        }
      }
      $exclude_id[] = $homePage;
    } elseif ($page == 1 && (in_array('page', $post_type))) {
      $available_pages[] = array('title' => 'Home Page', 'type' => '', 'id' => 'home', 'link' => home_url(), 'home' => true,);
    }

    if (!empty($postsPage)) {
      if ($page == 1 && (in_array('page', $post_type))) {
        if (!isset($post_info[$postsPage])) {
          $post_info[$postsPage] = $this->fetchPostInfo($postsPage, ' (Blog)');
        }
        if (!empty($search)) {
          if (stripos($post_info[$postsPage]['title'], $search) !== false) {
            $available_pages[] = $post_info[$postsPage];
          }
        } else {
          $available_pages[] = $post_info[$postsPage];
        }
      }
      $exclude_id[] = $postsPage;
    }


    $args = array('post_type' => $post_type, 'posts_per_page' => $limit, 'paged' => $page, 'offset' => $offset, 'orderby' => 'ID', 'order' => 'DESC', 'post_status' => 'publish', 'post__not_in' => $exclude_id, 'fields' => 'ids');

    if (!empty($search)) {
      $args = array_merge($args, ['s' => $search]);
    }

    $query = new WP_Query($args);
    $post_ids = $query->posts;

    $post_counter = 0;
    $update_transient = false;
    foreach ($post_ids as $post_id) {
      if (!isset($post_info[$post_id])) {
        $post_info[$post_id] = $this->fetchPostInfo($post_id);
        $update_transient = true;
      }

      $available_pages[] = $post_info[$post_id];

      // Add errors to the current page, if any
      if (isset($warmup_errors[$post_id])) {
        $available_pages[count($available_pages) - 1]['errors'] = $warmup_errors[$post_id];
      }

      $post_counter++;

      // Update the transient every 2000 posts in case the process dies
      if ($post_counter >= 2000 && $update_transient) {
        set_transient('wpc-post-info', $post_info, 3600);
        $post_counter = 0; // Reset counter
      }
    }

    if ($update_transient) {
      set_transient('wpc-post-info', $post_info, 3600);
    }

    return $available_pages;
  }

  private function fetchPostInfo($post_id, $suffix = '') {
    return array('title' => get_the_title($post_id) . $suffix, 'type' => get_post_type($post_id), 'id' => $post_id, 'link' => get_permalink($post_id));
  }

  public function getOptimizationsStatus($post_type = ['page', 'post'], $page = 1, $offset = 0, $limit = 10, $search = '') {
    $pages = $this->getPages($post_type, $page, $offset, $limit, $search);
    $url_key_class = new wps_ic_url_key();
    $wpc_excludes = get_option('wpc-excludes', array());
    $settings = get_option(WPS_IC_SETTINGS);

    //local addition
    $local = get_option('wpc-connectivity-status');
    //end local addition

    foreach ($pages as &$page) {
      if ($page['id'] == 'home') {
        $url = home_url();
      } else {
        $url = get_permalink($page['id']);
      }
      $urlKey = $url_key_class->setup($url);
      $cachePath = WPS_IC_CACHE . $urlKey . '/';
      $critPath = WPS_IC_CRITICAL . $urlKey . '/critical_desktop.css';
      $page_excludes = isset($wpc_excludes['page_excludes'][$page['id']]) ? $wpc_excludes['page_excludes'][$page['id']] : array();
      $cacheActive = (!empty($settings['cache']['advanced']) && $settings['cache']['advanced'] == '1' && !isset($page_excludes['advanced_cache'])) || (isset($page_excludes['advanced_cache']) && $page_excludes['advanced_cache'] == '1');

      $cacheGenerated = '0';
      $critGenerated = '1';
      if ($cacheActive) {
        if (function_exists('gzencode')) {
          if (file_exists($cachePath . 'index.html' . '_gzip') && filesize($cachePath . 'index.html' . '_gzip') > 0) {
            $cacheGenerated = '1';
          }
        } else {
          if (file_exists($cachePath . 'index.html') && filesize($cachePath . 'index.html') > 0) {
            $cacheGenerated = '1';
          }
        }
      }

      $doNotCache = false;
      if (!empty($page['errors']['notice'])) {
        $doNotCache = in_array('DONOTCACHEPAGE', $page['errors']['notice']);
      }

      //local addition
      if (!empty($local) && $local == 'failed' && ($cacheGenerated == '1' || $doNotCache)) {
        $preloaded = '1';
      } else {
        $preloaded = '1';

        //check critical
        if ((isset($page_excludes['critical_css']) && $page_excludes['critical_css'] == '0')) {
          // Excluded from Smart Optimizations
        } else if (!empty($settings['critical']['css']) && $settings['critical']['css'] == '1') {
          if (!file_exists($critPath)) {
            $critGenerated = '0';
            $preloaded = '0';
          }
        }

        if (($cacheGenerated == '0' && !$doNotCache) && $cacheActive) {
          $preloaded = '0';
        }
      }

      $exclude_array = $page_excludes;

      $tests = get_option('wpc-tests');

      if (!empty($tests[$urlKey])) {
        $test = $tests[$urlKey];
      } else {
        $test = 'false';
      }

      $running = '0';
      $runningBulk_t = get_transient('wpc-page-optimizations-status');
      if ($runningBulk_t === false) {
        $runningOther = '0';
      } else if ($runningBulk_t['id'] == $page['id']) {
        $running = '1';
      } else {
        $runningOther = '1';
      }

      global $wpdb;
      $transient_prefix = 'wpc_test_';
      $sql = $wpdb->prepare("SELECT option_name FROM {$wpdb->options} WHERE option_name LIKE %s", $wpdb->esc_like('_transient_' . $transient_prefix) . '%');
      $transient_keys = $wpdb->get_col($sql);
      foreach ($transient_keys as $transient_key) {
        $key = str_replace('_transient_', '', $transient_key);
        $value = get_transient($key);
      }


      if (!empty($transient_keys)) {
        if ($key === 'wpc_test_' . $page['id']) {
          $running = '1';
        } else {
          $runningOther = '1';
        }
      }


      $page = array_merge($page, array('cacheGenerated' => $cacheGenerated, 'critGenerated' => $critGenerated, 'preloaded' => $preloaded, 'test' => $test, 'running' => $running, 'runningOther' => $runningOther,), $exclude_array);
    }

    return $pages;
  }

  public function getPagesJSON() {
    header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
    header('Cache-Control: post-check=0, pre-check=0', false);
    header('Pragma: no-cache');
    header('Content-Type: application/json');

    if (!empty($_GET['apikey'])) {
      $apikey = sanitize_text_field($_GET['apikey']);
    } else {
      echo json_encode('no-apikey');
    }

    if (!empty($apikey) && get_option(WPS_IC_OPTIONS)['api_key'] == $apikey) {

      $pages = $this->getPagesToOptimize();

      if (empty($pages)) {
        echo json_encode('no-pages');
      }

      $page_links = [];
      foreach ($pages['pages'] as $page) {
        $generateCrit = 'false';
        if ($page['critGenerated'] == '0') {
          $generateCrit = 'true';
        }
        $page_links[$page['id']] = ['url' => $page['link'], 'critical' => $generateCrit];

        if (!empty($page['home'])) {
          $page_links[$page['id']]['test'] = 'true';
        }
      }

      echo json_encode(['pages' => $page_links]);
      die();
    }

    echo json_encode('no-apikey');
  }

  public function getPagesToOptimize() {
    $optimize = get_option('wpc-warmup-selector');
    if ($optimize === false) {
      $optimize = ['page'];
      update_option('wpc-warmup-selector', $optimize);
    } elseif ($optimize === 'do-not-optimize') {
      return [];
    }
    $pages = $this->getPages($optimize, 1, 0, -1);
    $wpc_excludes = get_option('wpc-excludes');
    $url_key_class = new wps_ic_url_key();
    $settings = get_option(WPS_IC_SETTINGS);

    //local addition
    $local = get_option('wpc-connectivity-status');
    //end local addition
    $return = [];
    $return['total'] = count($pages);
    foreach ($pages as $key => $page) {
      $url = $page['link'];
      $urlKey = $url_key_class->setup($url);
      $cachePath = WPS_IC_CACHE . $urlKey . '/';
      $critPath = WPS_IC_CRITICAL . $urlKey . '/critical_desktop.css';
      $page_excludes = isset($wpc_excludes['page_excludes'][$page['id']]) ? $wpc_excludes['page_excludes'][$page['id']] : array();
      $cacheActive = (!empty($settings['cache']['advanced']) && $settings['cache']['advanced'] == '1' && !isset($page_excludes['advanced_cache'])) || (isset($page_excludes['advanced_cache']) && $page_excludes['advanced_cache'] == '1');

      if (!empty($page['errors'])) {
        $hasErrorCode = false;
        foreach ($page['errors'] as $errorCode) {
          if (is_numeric($errorCode) && (int)$errorCode >= 300 && (int)$errorCode <= 600) {
            $hasErrorCode = true;
            break;
          }
        }

        if ($hasErrorCode) {
          $return['total'] = $return['total'] - 1;
          unset($pages[$key]);
          continue;
        }
      }

      $cacheGenerated = '0';
      $critGenerated = '1';
      if ($cacheActive) {
        if (function_exists('gzencode')) {
          if (file_exists($cachePath . 'index.html' . '_gzip') && filesize($cachePath . 'index.html' . '_gzip') > 0) {
            $cacheGenerated = '1';
          }
        } else {
          if (file_exists($cachePath . 'index.html') && filesize($cachePath . 'index.html') > 0) {
            $cacheGenerated = '1';
          }
        }
      }

      $doNotCache = false;
      if (!empty($page['errors']['notice'])) {
        $doNotCache = in_array('DONOTCACHEPAGE', $page['errors']['notice']);
      }

      //local addition
      if (!empty($local) && $local == 'failed' && ($cacheGenerated == '1' || $doNotCache)) {
        $preloaded = '1';
      } else {
        $preloaded = '1';

        //check critical
        if (!isset($page_excludes['critical_css']) || (isset($page_excludes['critical_css']) && $page_excludes['critical_css'] == '0')) {
          // Excluded in Smart Optimizations
        } else if (!empty($settings['critical']['css']) && $settings['critical']['css'] == '1') {
          if (!file_exists($critPath)) {
            $critGenerated = '0';
            $preloaded = '0';
          }
        }

        if (($cacheGenerated == '0' && !$doNotCache) && $cacheActive) {
          $preloaded = '0';
        }
      }


      $pages[$key]['critGenerated'] = $critGenerated;

      if (!empty($page['home'])) {
        $pages[$key]['home'] = true;
      }


      if ($preloaded == '1') {
        unset($pages[$key]);
      }
    }

    $return['unoptimized'] = count($pages);
    $return['pages'] = $pages;


    return $return;
  }

  public function downloadDesktopCrit() {
    header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
    header('Cache-Control: post-check=0, pre-check=0', false);
    header('Pragma: no-cache');
    header('Content-Type: application/json');

    if (!empty($_GET['apikey'])) {
      $apikey = sanitize_text_field($_GET['apikey']);
    } else {
      echo json_encode('no-apikey');
      die();
    }

    if (!empty($_GET['id'])) {
      $id = sanitize_text_field($_GET['id']);
    } else {
      echo json_encode('no-id');
      die();
    }

    if (!empty($_GET['desktopCritUrl'])) {
      $desktopCritUrl = esc_url($_GET['desktopCritUrl']);
    } else {
      echo json_encode('no-url');
      die();
    }

    if (get_option(WPS_IC_OPTIONS)['api_key'] == $apikey) {

      if ($id == 'home') {
        $url = home_url();
      } else {
        $url = get_permalink($id);
      }

      $criticalCSS = new wps_criticalCss($url);
      $criticalCSS->saveCriticalWarmup($id, $url, $desktopCritUrl, 'desktop');

      echo json_encode(['done']);
      die();
    } else {
      echo json_encode('wrong-apikey');
    }

  }

  public function downloadMobileCrit() {
    header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
    header('Cache-Control: post-check=0, pre-check=0', false);
    header('Pragma: no-cache');
    header('Content-Type: application/json');

    if (!empty($_GET['apikey'])) {
      $apikey = sanitize_text_field($_GET['apikey']);
    } else {
      echo json_encode('no-apikey');
      die();
    }

    if (!empty($_GET['id'])) {
      $id = sanitize_text_field($_GET['id']);
    } else {
      echo json_encode('no-id');
      die();
    }

    if (!empty($_GET['mobileCritUrl'])) {
      $desktopCritUrl = esc_url($_GET['mobileCritUrl']);
    } else {
      echo json_encode('no-url');
      die();
    }

    if (get_option(WPS_IC_OPTIONS)['api_key'] == $apikey) {

      if ($id == 'home') {
        $url = home_url();
      } else {
        $url = get_permalink($id);
      }

      $criticalCSS = new wps_criticalCss($url);
      $criticalCSS->saveCriticalWarmup($id, $url, $desktopCritUrl, 'mobile');

      echo json_encode(['done']);
      die();
    } else {
      echo json_encode('wrong-apikey');
    }

  }

  public function updateStatus() {
    header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
    header('Cache-Control: post-check=0, pre-check=0', false);
    header('Pragma: no-cache');
    header('Content-Type: application/json');

    if (!empty($_GET['apikey'])) {
      $apikey = sanitize_text_field($_GET['apikey']);
    } else {
      echo json_encode('no-apikey');
      die();
    }

    if (!empty($_GET['id'])) {
      $id = sanitize_text_field($_GET['id']);
    } else {
      echo json_encode('no-id');
      die();
    }

    if (get_option(WPS_IC_OPTIONS)['api_key'] == $apikey) {
      $oldStatus = get_transient('wpc-page-optimizations-status');
      $status = $this->isOptimized($id, true);
      $notOptimizedCount = 0;

      if (!empty($oldStatus['id'])) {
        $oldStatus = $this->isOptimized($oldStatus['id'], true);
        if ($oldStatus['optimized' == '0'] && empty($oldStatus['error_code'])) {
          if (!empty($oldStatus['notOptimizedCount'])) {
            $notOptimizedCount = $oldStatus['notOptimizedCount'];
          }
          $notOptimizedCount++;
        }
      }


      if ($notOptimizedCount >= 5) {
        $this->stopOptimizations();
        //write a notice, warning something
      }

      set_transient('wpc-page-optimizations-status', ['id' => $id, 'status' => 'warmup', 'notOptimizedCount' => $notOptimizedCount], 60 * 5);


      echo json_encode($status);
      die();
    } else {
      echo json_encode('wrong-apikey');
    }
  }

  public function isOptimized($id, $fullStatus = false) {
    $url_key_class = new wps_ic_url_key();
    $wpc_excludes = get_option('wpc-excludes', array());
    $settings = get_option(WPS_IC_SETTINGS);
    $errors = get_option('wpc-warmup-errors', []);

    //local addition
    $local = get_option('wpc-connectivity-status');
    //end local addition

    if ($id == 'home') {
      $url = home_url();
    } else {
      $url = get_permalink($id);
    }
    $urlKey = $url_key_class->setup($url);
    $cachePath = WPS_IC_CACHE . $urlKey . '/';
    $critPath = WPS_IC_CRITICAL . $urlKey . '/critical_desktop.css';
    $page_excludes = isset($wpc_excludes['page_excludes'][$id]) ? $wpc_excludes['page_excludes'][$id] : array();
    $cacheActive = (!empty($settings['cache']['advanced']) && $settings['cache']['advanced'] == '1' && !isset($page_excludes['advanced_cache'])) || (isset($page_excludes['advanced_cache']) && $page_excludes['advanced_cache'] == '1');

    $cacheGenerated = '0';
    $critGenerated = '1';
    if ($cacheActive) {
      if (function_exists('gzencode')) {
        if (file_exists($cachePath . 'index.html' . '_gzip') && filesize($cachePath . 'index.html' . '_gzip') > 0) {
          $cacheGenerated = '1';
        }
      } else {
        if (file_exists($cachePath . 'index.html') && filesize($cachePath . 'index.html') > 0) {
          $cacheGenerated = '1';
        }
      }
    }


    $doNotCache = false;
    if (!empty($page['errors']['notice'])) {
      $doNotCache = in_array('DONOTCACHEPAGE', $page['errors']['notice']);
    }

    $hasErrorCode = '0';
    if (!empty($page['errors'])) {
      foreach ($page['errors'] as $errorCode) {
        if (is_numeric($errorCode) && (int)$errorCode >= 300 && (int)$errorCode <= 600) {
          $hasErrorCode = '1';
        }
      }
    }

    //local addition
    if (!empty($local) && $local == 'failed' && ($cacheGenerated == '1' || $doNotCache || !$cacheActive)) {
      $preloaded = '1';
    } else {
      $preloaded = '1';


      //check critical
      if (isset($page_excludes['critical_css']) && $page_excludes['critical_css'] == '0') {
        // Exclude from Smart Optimizations
      } else if (!empty($settings['critical']['css']) && $settings['critical']['css'] == '1') {
        if (!file_exists($critPath)) {
          $critGenerated = '0';
          $preloaded = '0';
        }
      }

      if (($cacheGenerated == '0' && !$doNotCache) && $cacheActive) {
        $preloaded = '0';
      }

    }

    if (($cacheGenerated == '0' && !$doNotCache) && $cacheActive) {
      $preloaded = '0';
    }

    if ($fullStatus) {
      return ['critGenerated' => $critGenerated, 'preloaded' => $preloaded, 'error_code' => $hasErrorCode, 'cacheGenerated' => $cacheGenerated];
    }

    return $preloaded;
  }

  public function stopOptimizations() {
    $call = wp_remote_post(self::$apiUrl, ['method' => 'POST', 'sslverify' => false, 'user-agent' => 'Compress-API Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.182 Safari/537.36', 'body' => ['action' => 'stopOptimization', 'apikey' => get_option(WPS_IC_OPTIONS)['api_key']], 'timeout' => 10]);


    if (is_wp_error($call)) {
      wp_send_json_error($call->get_error_message());
    }

    if (wp_remote_retrieve_response_code($call) == 200) {
      $response_body = wp_remote_retrieve_body($call);
      $response_body = json_decode($response_body, true);
      if ($response_body['success'] == 'true') {
        delete_transient('wpc-page-optimizations-status');
        wp_send_json_success($call);
      } else {
        wp_send_json_error($call);
      }
    } else {
      wp_send_json_error(print_r($call, true));
    }
  }

  public function deliverError() {
    header('Cache-Control: no-store, no-cache, must-revalidate, max-age=0');
    header('Cache-Control: post-check=0, pre-check=0', false);
    header('Pragma: no-cache');
    header('Content-Type: application/json');

    if (!empty($_GET['apikey'])) {
      $apikey = sanitize_text_field($_GET['apikey']);
    } else {
      echo json_encode('no-apikey');
      die();
    }

    if (!empty($_GET['id'])) {
      $id = sanitize_text_field($_GET['id']);
    } else {
      echo json_encode('no-id');
      die();
    }

    if (!empty($_GET['errorCode'])) {
      $errorCode = sanitize_text_field($_GET['errorCode']);
    } else {
      echo json_encode('no-error');
      die();
    }

    if (get_option(WPS_IC_OPTIONS)['api_key'] == $apikey) {

      $this->addError($id, $errorCode, 'skip');

      echo json_encode(['done']);
      die();
    } else {
      echo json_encode('wrong-apikey');
    }
  }

  public function addError($id, $error, $type = 'notice') {
    $option_key = 'wpc-warmup-errors';
    $new_value = $error;

    $current_errors = get_option($option_key, []);

    if ($type == 'skip') {
      $current_errors[$id][$type] = $error;
    } else {
      if (isset($current_errors[$id][$type])) {
        if (!in_array($new_value, $current_errors[$id][$type])) {
          $current_errors[$id][$type][] = $new_value;
        }
      } else {
        $current_errors[$id][$type] = [];
        $current_errors[$id][$type][] = $new_value;
      }
    }

    update_option($option_key, $current_errors);
  }

  public function optimizeSingle($id, $test = true) {

    if ($id == 'home') {
      $url = home_url();
    } else {
      $url = get_permalink($id);
    }

    $page_links[$id] = ['url' => $url, 'test' => $test];

    $call = wp_remote_post(self::$apiUrl, ['method' => 'POST', 'sslverify' => false, 'user-agent' => 'Compress-API Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.182 Safari/537.36', 'body' => ['action' => 'createQueue', 'pages' => json_encode($page_links), 'apikey' => get_option(WPS_IC_OPTIONS)['api_key']], 'timeout' => 10]);

    if (is_wp_error($call)) {
      wp_send_json_error($call->get_error_message());
    }

    if (wp_remote_retrieve_response_code($call) == 200) {
      $response_body = wp_remote_retrieve_body($call);
      $response_body = json_decode($response_body, true);
      if ($response_body['success'] == 'true') {
        set_transient('wpc-page-optimizations-status', ['id' => 0, 'status' => 'started'], 60 * 3);
        wp_send_json_success($response_body['data']);
      } else {
        wp_send_json_error(print_r($response_body['data'], true));
      }
    } else {
      wp_send_json_error(print_r($call, true));
    }
  }

  public function doTest($id, $retest = false) {
    if ($id == 'home') {
      $url = home_url();
    } else {
      $url = get_permalink($id);
    }
    $url_key_class = new wps_ic_url_key();
    $urlKey = $url_key_class->setup($url);

    set_transient('wpc_test_' . $id, 'started', 60);

    $results = get_option('wpc-tests', []);
    if ($retest === false) {
      if (!empty($results[$urlKey])) {
        wp_send_json_success($results[$urlKey]);
      }
    } else {
      $results[$urlKey] = [];
      update_option('wpc-tests', $results);
    }

    $call = wp_remote_post(self::$apiUrl, ['timeout' => 100, 'blocking' => true, 'body' => ['id' => $id, 'url' => $url, 'apikey' => get_option(WPS_IC_OPTIONS)['api_key'], 'action' => 'doTest'], 'sslverify' => false, 'user-agent' => 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:20.0) Gecko/20100101 Firefox/20.0']);

    delete_transient('wpc_test_' . $id);

    if (wp_remote_retrieve_response_code($call) == 200) {
      $body = wp_remote_retrieve_body($call);
      $decodedBody = json_decode($body, true);

      if (!empty($decodedBody['success']) && $decodedBody['success'] == 'true') {
        // Update the option with new results
        $results = get_option('wpc-tests', []);
        $results[$urlKey] = $decodedBody['data'];
        update_option('wpc-tests', $results);
        wp_send_json_success($decodedBody['data']);
      }
    }

    wp_send_json_error(array(self::$apiUrl, ['id' => $id, 'url' => $url, 'apikey' => get_option(WPS_IC_OPTIONS)['api_key'], 'action' => 'doTest']), $call);
  }

  public function get_optimization_status() {

    $status = get_transient('wpc-page-optimizations-status');
    $connectivity = get_transient('wpc-connectivity-status');
    if ($status !== false) {
      $status['pageTitle'] = ($status['id'] === 'home') ? 'Home Page' : get_the_title($status['id']);
      if (!empty($connectivity) && $connectivity == 'failed') {
        $status['mode'] = 'local';
      }
    }
    return $status;

  }

  public function connectivityTest() {
    $url = home_url();
    $api_key = get_option(WPS_IC_OPTIONS)['api_key'];
    $results = [];

    // Function to process the response
    $processResponse = function ($response) {
      if (is_wp_error($response)) {
        return ['success' => false, 'error' => $response->get_error_message()];
      }

      $body = wp_remote_retrieve_body($response);
      $code = wp_remote_retrieve_response_code($response);

      if ($code != 200) {
        return ['success' => false, 'error' => "HTTP status code: $code"];
      }

      return ['success' => true, 'response' => json_decode($body, true)];
    };

    // Test POST request without params
    try {
      $postNoParamsResponse = wp_remote_post(self::$apiUrl);
      $results['post_no_params'] = $processResponse($postNoParamsResponse);
    } catch (Exception $e) {
      $results['post_no_params'] = ['success' => false, 'error' => 'Error: ' . $e->getMessage()];
    }

    // Test POST request with params
    try {
      $postWithParamsResponse = wp_remote_post(self::$apiUrl, ['body' => ['apikey' => $api_key, 'url' => $url]]);
      $results['post_with_params'] = $processResponse($postWithParamsResponse);
    } catch (Exception $e) {
      $results['post_with_params'] = ['success' => false, 'error' => 'Error: ' . $e->getMessage()];
    }

    // Test GET request without params
    try {
      $getNoParamsResponse = wp_remote_get(self::$apiUrl);
      $results['get_no_params'] = $processResponse($getNoParamsResponse);
    } catch (Exception $e) {
      $results['get_no_params'] = ['success' => false, 'error' => 'Error: ' . $e->getMessage()];
    }

    // Test GET request with params
    try {
      $getWithParamsResponse = wp_remote_get(self::$apiUrl . '?apikey=' . urlencode($api_key) . '&url=' . urlencode($url));
      $results['get_with_params'] = $processResponse($getWithParamsResponse);
    } catch (Exception $e) {
      $results['get_with_params'] = ['success' => false, 'error' => 'Error: ' . $e->getMessage()];
    }

    // Test GET request with params in headers
    try {
      $getWithHeadersResponse = wp_remote_get(self::$apiUrl, ['headers' => ['apikey' => $api_key, 'url' => $url]]);
      $results['get_with_headers'] = $processResponse($getWithHeadersResponse);
    } catch (Exception $e) {
      $results['get_with_headers'] = ['success' => false, 'error' => 'Error: ' . $e->getMessage()];
    }

    // Determine the best working method
    $methodToUse = null;
    if ($results['get_with_params']['success']) {
      $methodToUse = 'get_with_params';
    } elseif ($results['post_with_params']['success']) {
      $methodToUse = 'post_with_params';
    } elseif ($results['get_with_headers']['success']) {
      $methodToUse = 'get_with_headers';
    }

    // If a method worked, send the connectivity test request
    if ($methodToUse) {
      $actionParams = ['apikey' => $api_key, 'url' => $url, 'action' => 'connectivityTest'];

      switch ($methodToUse) {
        case 'get_with_params':
          $finalTestResponse = wp_remote_get(self::$apiUrl . '?' . http_build_query($actionParams));
          break;
        case 'post_with_params':
          $finalTestResponse = wp_remote_post(self::$apiUrl, ['body' => $actionParams]);
          break;
        case 'get_with_headers':
          $finalTestResponse = wp_remote_get(self::$apiUrl, ['headers' => $actionParams]);
          break;
      }

      $results['final_test'] = $processResponse($finalTestResponse);
    }

    // Save the result to a transient
    set_transient('api_test_results', $results, 60 * 5); // 5 minutes expiration

    return $results;
  }

  function setupCronPreload() {
    add_action('init', [$this, 'scheduleCronWarmup']);
    add_action('runCronPreload', [$this, 'startOptimizationsCron']);
    add_filter('cron_schedules', [$this, 'setupPreloadInterval']);
  }

  function scheduleCronWarmup() {
    $timestamp = wp_next_scheduled('runCronPreload');
    if ($timestamp) {
    } else {
      // If the job isn't scheduled, schedule it
      wp_schedule_event(time(), 'every_three_hours', 'runCronPreload');
    }
  }

  function setupPreloadInterval($schedules) {
    $schedules['every_three_hours'] = array('interval' => 10800, 'display' => esc_html__('Every Three Hours'),);
    return $schedules;
  }

  public function startOptimizationsCron() {
    //$shouldStart = get_option('wps_ic_start_cron'); //or whatever the name of the setting from toggle will be
    //if (!empty($shouldStart) && $shouldStart === 'true'){
    $this->startOptimizations();
    //}
  }

  public function startOptimizations() {

    $this->simpleConnectivityTest();
    $connectivity = get_option('wpc-connectivity-status');
    if (!empty($connectivity) && $connectivity == 'failed') {
      //engage local mode
      set_transient('wpc-page-optimizations-status', ['id' => '', 'status' => 'started', 'mode' => 'local'], 60 * 3);
      wp_send_json_success();
    } else {
      update_option('wpc-connectivity-status', 'passed');
    }

    $call = wp_remote_post(self::$apiUrl, ['method' => 'POST', 'sslverify' => false, 'user-agent' => 'Compress-API Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.182 Safari/537.36', 'body' => ['action' => 'getPagesJSON', 'apikey' => get_option(WPS_IC_OPTIONS)['api_key']], 'timeout' => 10]);


    if (is_wp_error($call)) {
      wp_send_json_error($call->get_error_message());
    }

    if (wp_remote_retrieve_response_code($call) == 200) {
      $response_body = wp_remote_retrieve_body($call);
      $response_body = json_decode($response_body, true);
      if ($response_body['success'] == 'true') {
        set_transient('wpc-page-optimizations-status', ['id' => '', 'status' => 'started'], 60 * 3);
        wp_send_json_success($response_body['data']);
      } else {
        wp_send_json_error(print_r($response_body['data'], true));
      }
    } else {
      wp_send_json_error(print_r($call, true));
    }
  }

  public function simpleConnectivityTest() {
    //only test post with params outbound and get with params inbound, what we are using
    $url = home_url();
    $api_key = get_option(WPS_IC_OPTIONS)['api_key'];
    $results = [];

    // Function to process the response
    $processResponse = function ($response) {
      if (is_wp_error($response)) {
        return ['success' => false, 'error' => $response->get_error_message()];
      }

      $body = wp_remote_retrieve_body($response);
      $code = wp_remote_retrieve_response_code($response);

      if ($code != 200) {
        return ['success' => false, 'error' => "HTTP status code: $code"];
      }

      return ['success' => true, 'response' => json_decode($body, true)];
    };

    // Test POST request with params
    try {
      $postWithParamsResponse = wp_remote_post(self::$apiUrl, ['body' => ['apikey' => $api_key, 'url' => $url]]);
      $results['post_with_params'] = $processResponse($postWithParamsResponse);
    } catch (Exception $e) {
      $results['post_with_params'] = ['success' => false, 'error' => 'Error: ' . $e->getMessage()];
    }


    // Determine the best working method
    $methodToUse = null;
    if ($results['post_with_params']['success']) {
      $methodToUse = 'post_with_params';
    }
    // If a method worked, send the connectivity test request
    if ($methodToUse) {
      $actionParams = ['apikey' => $api_key, 'url' => $url, 'action' => 'simpleConnectivityTest'];

      $finalTestResponse = wp_remote_post(self::$apiUrl, ['body' => $actionParams]);
      $results['final_test'] = $processResponse($finalTestResponse);
    }

    $outbound = !empty($results['post_with_params']['response']['data']) && $results['post_with_params']['response']['data'] == 'missing-action';
    $inbound = !empty($results['final_test']['response']['data']['get_with_params']['response']) && (strpos($results['final_test']['response']['data']['get_with_params']['response'], 'passed') !== false);
    if (!$inbound || !$outbound) {
      update_option('wpc-connectivity-status', 'failed');
    } else {
      update_option('wpc-connectivity-status', 'passed');
    }
  }

  function isRedirected($url) {
    $args = array('method' => 'HEAD', 'redirection' => 0, 'timeout' => 5);

    $response = wp_remote_request($url, $args);

    if (is_wp_error($response)) {
      return false;
    }

    $response_code = wp_remote_retrieve_response_code($response);

    // Check if the response code is a 3xx (redirection)
    if ($response_code >= 300 && $response_code < 400) {
      return true;
    }

    return false;
  }

}